@@ -387,7 +387,7 @@ class FormWithActsLikeFormForTest < FormWithTest
 
     form_with(model: post_form) { }
 
-    expected = whole_form("/posts/123", method: :post) { "" }
+    expected = whole_form("/posts/123", method: :patch) { "" }
 
     assert_dom_equal expected, @rendered
   end
@@ -1770,14 +1770,20 @@ class FormWithActsLikeFormForTest < FormWithTest
   def test_nested_fields_label_translation_with_more_than_10_records
     @post.comments = Array.new(11) { |id| Comment.new(id + 1) }
 
-    params = 11.times.map { ["post.comments.body", default: [:"comment.body", ""], scope: "helpers.label"] }
-    assert_called_with(I18n, :t, params, returns: "Write body here") do
+    mock = Minitest::Mock.new
+    @post.comments.each do
+      expect_called_with(mock, ["post.comments.body"], default: [:"comment.body", ""], scope: "helpers.label", returns: "body")
+    end
+
+    I18n.stub(:t, mock) do
       form_with(model: @post) do |f|
         f.fields(:comments) do |cf|
           concat cf.label(:body)
         end
       end
     end
+
+    assert_mock(mock)
   end
 
   def test_nested_fields_with_existing_records_on_a_supplied_nested_attributes_collection_different_from_record_one
@@ -1994,6 +2000,22 @@ class FormWithActsLikeFormForTest < FormWithTest
     assert_dom_equal expected, @rendered
   end
 
+  def test_nested_fields_with_hash_with_indifferent_access_like_model
+    @author = HashWithIndifferentAccessBackedAuthor.new
+
+    form_with(model: @post) do |f|
+      concat f.fields(:author, model: @author) { |af|
+        concat af.text_field(:name)
+      }
+    end
+
+    expected = whole_form("/posts/123", method: "patch") do
+      '<input name="post[author_attributes][name]" type="text" value="hash backed author" id="post_author_attributes_name" />'
+    end
+
+    assert_dom_equal expected, @rendered
+  end
+
   def test_fields
     @rendered = fields(:post, model: @post) do |f|
       concat f.text_field(:title)
