@@ -41,6 +41,18 @@ module ActiveRecord
       assert_not_predicate deferred_posts, :scheduled?
     end
 
+    def test_null_scheduled?
+      deferred_null_posts = Post.none.load_async
+      if in_memory_db?
+        assert_not_predicate deferred_null_posts, :scheduled?
+      else
+        assert_predicate deferred_null_posts, :scheduled?
+      end
+      assert_predicate deferred_null_posts, :loaded?
+      deferred_null_posts.to_a
+      assert_not_predicate deferred_null_posts, :scheduled?
+    end
+
     def test_reset
       deferred_posts = Post.where(author_id: 1).load_async
       if in_memory_db?
@@ -220,10 +232,17 @@ module ActiveRecord
       assert_equal false, deferred_posts.empty?
       assert_predicate deferred_posts, :loaded?
     end
+
+    def test_load_async_with_query_cache
+      titles = Post.where(author_id: 1).pluck(:title)
+      Post.cache do
+        assert_equal titles, Post.where(author_id: 1).load_async.pluck(:title)
+      end
+    end
   end
 
-  unless in_memory_db?
-    class LoadAsyncNullExecutorTest < ActiveRecord::TestCase
+  class LoadAsyncNullExecutorTest < ActiveRecord::TestCase
+    unless in_memory_db?
       self.use_transactional_tests = false
 
       fixtures :posts, :comments
@@ -231,12 +250,10 @@ module ActiveRecord
       def setup
         @old_config = ActiveRecord.async_query_executor
         ActiveRecord.async_query_executor = nil
-        ActiveRecord::Base.establish_connection :arunit
       end
 
       def teardown
         ActiveRecord.async_query_executor = @old_config
-        ActiveRecord::Base.establish_connection :arunit
       end
 
       def test_scheduled?
@@ -344,8 +361,10 @@ module ActiveRecord
         assert_predicate deferred_posts, :loaded?
       end
     end
+  end
 
-    class LoadAsyncMultiThreadPoolExecutorTest < ActiveRecord::TestCase
+  class LoadAsyncMultiThreadPoolExecutorTest < ActiveRecord::TestCase
+    unless in_memory_db?
       include WaitForAsyncTestHelper
 
       self.use_transactional_tests = false
@@ -356,22 +375,28 @@ module ActiveRecord
         @old_config = ActiveRecord.async_query_executor
         ActiveRecord.async_query_executor = :multi_thread_pool
 
-        handler = ActiveRecord::ConnectionAdapters::ConnectionHandler.new
-        config_hash1 = ActiveRecord::Base.configurations.configs_for(env_name: "arunit", name: "primary").configuration_hash
-        new_config1 = config_hash1.merge(min_threads: 0, max_threads: 10)
-        db_config1 = ActiveRecord::DatabaseConfigurations::HashConfig.new("arunit", "primary", new_config1)
+        config_hash1 = ActiveRecord::Base.connection_pool.db_config.configuration_hash.merge(min_threads: 0, max_threads: 10)
+        config_hash2 = ARUnit2Model.connection_pool.db_config.configuration_hash.merge(min_threads: 0, max_threads: 10)
 
-        config_hash2 = ActiveRecord::Base.configurations.configs_for(env_name: "arunit2", name: "primary").configuration_hash
-        new_config2 = config_hash2.merge(min_threads: 0, max_threads: 10)
-        db_config2 = ActiveRecord::DatabaseConfigurations::HashConfig.new("arunit2", "primary", new_config2)
-
-        handler.establish_connection(db_config1)
-        handler.establish_connection(db_config2, owner_name: ARUnit2Model)
+        ActiveRecord::Base.establish_connection(config_hash1)
+        ARUnit2Model.establish_connection(config_hash2)
       end
 
       def teardown
         ActiveRecord.async_query_executor = @old_config
         clean_up_connection_handler
+        ActiveRecord::Base.establish_connection(:arunit)
+        ARUnit2Model.establish_connection(:arunit2)
+      end
+
+      def test_async_query_executor_and_configuration
+        assert_equal :multi_thread_pool, ActiveRecord.async_query_executor
+
+        assert_equal 0, ActiveRecord::Base.connection_pool.db_config.configuration_hash[:min_threads]
+        assert_equal 0, ARUnit2Model.connection_pool.db_config.configuration_hash[:min_threads]
+
+        assert_equal 10, ActiveRecord::Base.connection_pool.db_config.configuration_hash[:max_threads]
+        assert_equal 10, ARUnit2Model.connection_pool.db_config.configuration_hash[:max_threads]
       end
 
       def test_scheduled?
@@ -477,8 +502,10 @@ module ActiveRecord
         assert_predicate deferred_posts, :loaded?
       end
     end
+  end
 
-    class LoadAsyncMixedThreadPoolExecutorTest < ActiveRecord::TestCase
+  class LoadAsyncMixedThreadPoolExecutorTest < ActiveRecord::TestCase
+    unless in_memory_db?
       include WaitForAsyncTestHelper
 
       self.use_transactional_tests = false
