@@ -101,7 +101,7 @@ class TimeWithZoneTest < ActiveSupport::TestCase
     with_zone = ActiveSupport::TimeWithZone.new(nil, ActiveSupport::TimeZone["Hawaii"], local)
 
     assert_equal local.nsec, with_zone.nsec
-    assert_equal with_zone.nsec, 999999999
+    assert_equal 999999999, with_zone.nsec
   end
 
   def test_strftime
@@ -133,16 +133,6 @@ class TimeWithZoneTest < ActiveSupport::TestCase
     assert_equal "1999-12-31 19:00:00 -0500", @twz.to_s
   end
 
-  def test_to_s_default_is_deprecated
-    Time::DATE_FORMATS[:default] = "%Y/%m/%d"
-
-    assert_deprecated do
-      assert_equal "1999/12/31", @twz.to_s
-    end
-  ensure
-    Time::DATE_FORMATS.delete(:default)
-  end
-
   def test_to_fs
     assert_equal "1999-12-31 19:00:00 -0500", @twz.to_fs
   end
@@ -156,22 +146,8 @@ class TimeWithZoneTest < ActiveSupport::TestCase
     assert_equal "1999-12-31 19:00:00.000000000 -0500", @twz.to_fs(:inspect)
   end
 
-  def test_to_s_db
-    assert_deprecated do
-      assert_equal "2000-01-01 00:00:00", @twz.to_s(:db)
-    end
-  end
-
-  def test_to_s_inspect
-    assert_deprecated do
-      assert_equal "1999-12-31 19:00:00.000000000 -0500", @twz.to_s(:inspect)
-    end
-  end
-
-  def test_to_s_not_existent
-    assert_deprecated do
-      assert_equal "1999-12-31 19:00:00 -0500", @twz.to_s(:not_existent)
-    end
+  def test_to_fs_not_existent
+    assert_equal "1999-12-31 19:00:00 -0500", @twz.to_fs(:not_existent)
   end
 
   def test_xmlschema
@@ -216,7 +192,7 @@ class TimeWithZoneTest < ActiveSupport::TestCase
     EOF
 
     # TODO: Remove assertion in Rails 7.1
-    assert_not_deprecated do
+    assert_not_deprecated(ActiveSupport.deprecator) do
       assert_equal(yaml, @twz.to_yaml)
     end
   end
@@ -232,7 +208,7 @@ class TimeWithZoneTest < ActiveSupport::TestCase
     EOF
 
     # TODO: Remove assertion in Rails 7.1
-    assert_not_deprecated do
+    assert_not_deprecated(ActiveSupport.deprecator) do
       assert_equal(yaml, { "twz" => @twz }.to_yaml)
     end
   end
@@ -603,13 +579,6 @@ class TimeWithZoneTest < ActiveSupport::TestCase
     assert_kind_of ActiveSupport::TimeWithZone, @twz
   end
 
-  def test_class_name
-    # TODO: Remove assertion in Rails 7.1 and change expected value
-    assert_deprecated("ActiveSupport::TimeWithZone.name has been deprecated") do
-      assert_equal "Time", ActiveSupport::TimeWithZone.name
-    end
-  end
-
   def test_method_missing_with_time_return_value
     assert_instance_of ActiveSupport::TimeWithZone, @twz.months_since(1)
     assert_equal Time.utc(2000, 1, 31, 19, 0, 0), @twz.months_since(1).time
@@ -1108,12 +1077,10 @@ class TimeWithZoneTest < ActiveSupport::TestCase
   end
 
   def test_no_method_error_has_proper_context
-    rubinius_skip "Error message inconsistency"
-
     e = assert_raises(NoMethodError) {
       @twz.this_method_does_not_exist
     }
-    assert_match(/undefined method `this_method_does_not_exist' for.*ActiveSupport::TimeWithZone/, e.message)
+    assert_match(/undefined method [`']this_method_does_not_exist' for.*ActiveSupport::TimeWithZone/, e.message)
     assert_no_match "rescue", e.backtrace.first
   end
 end
