@@ -12,21 +12,20 @@ require "action_view"
 
 require "rails_guides/markdown"
 require "rails_guides/helpers"
-require "rails_guides/levenshtein"
 require "rails_guides/epub"
 
 module RailsGuides
   class Generator
     GUIDES_RE = /\.(?:erb|md)\z/
 
-    def initialize(edge:, version:, all:, only:, epub:, language:, direction: "ltr")
+    def initialize(edge:, version:, all:, only:, epub:, language:, direction: nil)
       @edge      = edge
       @version   = version
       @all       = all
       @only      = only
       @epub      = epub
       @language  = language
-      @direction = direction
+      @direction = direction || "ltr"
 
       if @epub
         register_special_mime_types
@@ -108,14 +107,6 @@ module RailsGuides
 
       def copy_assets
         FileUtils.cp_r(Dir.glob("#{@guides_dir}/assets/*"), @output_dir)
-
-        if @direction == "rtl"
-          overwrite_css_with_right_to_left_direction
-        end
-      end
-
-      def overwrite_css_with_right_to_left_direction
-        FileUtils.mv("#{@output_dir}/stylesheets/main.rtl.css", "#{@output_dir}/stylesheets/main.css")
       end
 
       def output_file_for(guide)
@@ -186,7 +177,7 @@ module RailsGuides
         anchors = Set.new
         html.scan(/<h\d\s+id="([^"]+)/).flatten.each do |anchor|
           if anchors.member?(anchor)
-            puts "*** DUPLICATE ID: #{anchor}, please make sure that there're no headings with the same name at the same level."
+            puts "*** DUPLICATE ID: #{anchor}, please make sure that there are no headings with the same name at the same level."
           else
             anchors << anchor
           end
@@ -202,9 +193,7 @@ module RailsGuides
         html.scan(/<a\s+href="#([^"]+)/).flatten.each do |fragment_identifier|
           next if fragment_identifier == "mainCol" # in layout, jumps to some DIV
           unless anchors.member?(CGI.unescape(fragment_identifier))
-            guess = anchors.min { |a, b|
-              Levenshtein.distance(fragment_identifier, a) <=> Levenshtein.distance(fragment_identifier, b)
-            }
+            guess = DidYouMean::SpellChecker.new(dictionary: anchors).correct(fragment_identifier).first
             puts "*** BROKEN LINK: ##{fragment_identifier}, perhaps you meant ##{guess}."
           end
         end
