@@ -1,11 +1,14 @@
 # frozen_string_literal: true
 
 require "shellwords"
+require "env_helpers"
 
 #
 # Tests, setup, and teardown common to the application and plugin generator suites.
 #
 module SharedGeneratorTests
+  include EnvHelpers
+
   def setup
     Rails.application = TestApp::Application
     super
@@ -28,6 +31,24 @@ module SharedGeneratorTests
     destination_root
   end
 
+  def test_implied_options
+    generator([destination_root], ["--skip-active-job"])
+
+    assert_option :skip_action_mailer
+    assert_option :skip_active_storage
+    assert_option :skip_action_mailbox
+    assert_option :skip_action_text
+    assert_not_option :skip_active_record
+  end
+
+  def test_implied_options_with_conflicting_option
+    error = assert_raises do
+      run_generator [destination_root, "--skip-active-job", "--no-skip-active-storage"]
+    end
+
+    assert_match %r/conflicting option/i, error.message
+  end
+
   def test_skeleton_is_created
     run_generator
 
@@ -74,6 +95,42 @@ module SharedGeneratorTests
     assert_file "bin/rails", /#!\/usr\/bin\/env/
   end
 
+  def test_template_from_absolute_path
+    assert_match "It works from file!", run_generator([destination_root, "-m", "#{fixtures_root}/lib/template.rb"])
+  end
+
+  def test_template_from_relative_path
+    FileUtils.cd(fixtures_root)
+    assert_match "It works from file!", run_generator([destination_root, "-m", "lib/template.rb"])
+  end
+
+  def test_template_with_env_var_in_path
+    switch_env "FIXTURES_ROOT", fixtures_root do
+      assert_match "It works from file!", run_generator([destination_root, "-m", "$FIXTURES_ROOT/lib/template.rb"])
+    end
+  end
+
+  def test_template_with_curly_brace_env_var_in_path
+    switch_env "FIXTURES_ROOT", fixtures_root do
+      assert_match "It works from file!", run_generator([destination_root, "-m", "${FIXTURES_ROOT}/lib/template.rb"])
+    end
+  end
+
+  def test_template_with_windows_env_var_in_path
+    switch_env "FIXTURES_ROOT", fixtures_root do
+      assert_match "It works from file!", run_generator([destination_root, "-m", "%FIXTURES_ROOT%/lib/template.rb"])
+    end
+  end
+
+  def test_template_with_special_characters_in_path
+    Dir.mktmpdir do |dir|
+      template_path = "#{dir}/company's $99 template  (original).rb"
+      FileUtils.cp "#{fixtures_root}/lib/template.rb", template_path
+
+      assert_match "It works from file!", run_generator([destination_root, "-m", template_path])
+    end
+  end
+
   def test_template_raises_an_error_with_invalid_path
     quietly do
       content = capture(:stderr) { run_generator([destination_root, "-m", "non/existent/path"]) }
@@ -100,6 +157,7 @@ module SharedGeneratorTests
   def test_skip_git
     run_generator [destination_root, "--skip-git", "--full"]
     assert_no_file(".gitignore")
+    assert_no_file(".gitattributes")
     assert_no_directory(".git")
   end
 
@@ -155,22 +213,6 @@ module SharedGeneratorTests
     end
   end
 
-  def test_gitignore_when_sqlite3
-    run_generator
-
-    assert_file ".gitignore" do |content|
-      assert_match(/sqlite3/, content)
-    end
-  end
-
-  def test_gitignore_when_non_sqlite3_db
-    run_generator([destination_root, "-d", "mysql"])
-
-    assert_file ".gitignore" do |content|
-      assert_no_match(/sqlite/i, content)
-    end
-  end
-
   def test_generator_if_skip_active_record_is_given
     run_generator [destination_root, "--skip-active-record"]
     assert_no_directory "#{application_path}/db/"
@@ -183,9 +225,6 @@ module SharedGeneratorTests
     assert_file "#{application_path}/bin/setup" do |setup_content|
       assert_no_match(/db:prepare/, setup_content)
     end
-    assert_file ".gitignore" do |content|
-      assert_no_match(/sqlite/i, content)
-    end
   end
 
   def test_generator_for_active_storage
@@ -206,10 +245,6 @@ module SharedGeneratorTests
     assert_file "#{application_path}/config/storage.yml"
     assert_directory "#{application_path}/storage"
     assert_directory "#{application_path}/tmp/storage"
-
-    assert_file ".gitignore" do |content|
-      assert_match(/\/storage\//, content)
-    end
   end
 
   def test_generator_if_skip_active_storage_is_given
@@ -230,12 +265,6 @@ module SharedGeneratorTests
     end
 
     assert_no_file "#{application_path}/config/storage.yml"
-    assert_no_directory "#{application_path}/storage"
-    assert_no_directory "#{application_path}/tmp/storage"
-
-    assert_file ".gitignore" do |content|
-      assert_no_match(/\/storage\//, content)
-    end
   end
 
   def test_generator_does_not_generate_active_storage_contents_if_skip_active_record_is_given
@@ -256,12 +285,6 @@ module SharedGeneratorTests
     end
 
     assert_no_file "#{application_path}/config/storage.yml"
-    assert_no_directory "#{application_path}/storage"
-    assert_no_directory "#{application_path}/tmp/storage"
-
-    assert_file ".gitignore" do |content|
-      assert_no_match(/\/storage\//, content)
-    end
   end
 
   def test_generator_if_skip_action_mailer_is_given
@@ -278,6 +301,7 @@ module SharedGeneratorTests
     end
     assert_no_directory "#{application_path}/app/mailers"
     assert_no_directory "#{application_path}/test/mailers"
+    assert_no_file "#{application_path}/app/views/layouts/mailer.html.erb"
   end
 
   def test_generator_if_skip_action_cable_is_given
@@ -349,6 +373,10 @@ module SharedGeneratorTests
   end
 
   private
+    def fixtures_root
+      File.expand_path("../fixtures", __dir__)
+    end
+
     def run_generator_instance
       @bundle_commands = []
       @bundle_command_stub ||= -> (command, *) { @bundle_commands << command }
@@ -365,12 +393,18 @@ module SharedGeneratorTests
 
       generator(positional_args, option_args)
 
+      prerelease_commands = []
+      prerelease_command_rails_gems = []
       rails_gem_pattern = /^gem ["']rails["'], .+/
-      bundle_command_rails_gems = []
+
       @bundle_command_stub = -> (command, *) do
         @bundle_commands << command
-        assert_file File.expand_path("Gemfile", project_path) do |gemfile|
-          bundle_command_rails_gems << gemfile[rails_gem_pattern]
+
+        if command.start_with?("install", "exec rails")
+          prerelease_commands << command
+          assert_file File.expand_path("Gemfile", project_path) do |gemfile|
+            prerelease_command_rails_gems << gemfile[rails_gem_pattern]
+          end
         end
       end
 
@@ -380,16 +414,29 @@ module SharedGeneratorTests
       end
 
       assert_file File.expand_path("Gemfile", project_path) do |gemfile|
-        assert_equal "install", @bundle_commands[0]
-        assert_equal gemfile[rails_gem_pattern], bundle_command_rails_gems[0]
+        assert_equal "install", prerelease_commands[0]
+        assert_equal gemfile[rails_gem_pattern], prerelease_command_rails_gems[0]
 
-        assert_match %r"^exec rails (?:plugin )?new #{Regexp.escape Shellwords.join(expected_args)}", @bundle_commands[1]
-        assert_equal gemfile[rails_gem_pattern], bundle_command_rails_gems[1]
+        assert_match %r"^exec rails (?:plugin )?new #{Regexp.escape Shellwords.join(expected_args)}", prerelease_commands[1]
+        assert_equal gemfile[rails_gem_pattern], prerelease_command_rails_gems[1]
       end
     end
 
+    def assert_option(option)
+      assert generator.options[option], "Expected generator option #{option.inspect} to be truthy."
+    end
+
+    def assert_not_option(option)
+      assert_not generator.options[option], "Expected generator option #{option.inspect} to be falsy."
+    end
+
     def assert_gem(name, constraint = nil)
-      constraint_pattern = /, #{Regexp.escape constraint}/ if constraint
+      constraint_pattern =
+        if constraint.is_a?(String)
+          /, #{Regexp.escape constraint}/
+        elsif constraint
+          /, #{constraint}/
+        end
       assert_file "Gemfile", %r/^\s*gem ["']#{name}["']#{constraint_pattern}/
     end
 
